#![allow(dead_code)]
use solana_client::rpc_client::RpcClient;
use solana_sdk::pubkey::Pubkey;
use solana_sdk::signature::keypair::Keypair;
use solana_sdk::signer::Signer;
use tokio::time::{sleep, Duration};

const RPC_URL: &str = "https://api.devnet.solana.com";

pub struct Client {
    client: RpcClient,
    keypair: Keypair,
    token: Pubkey,
}

impl Client {
    pub async fn new() -> Self {
        let client = RpcClient::new(RPC_URL);
        let keypair = Keypair::new();
        let token = create_token(&client, &keypair).await;
        Self {
            client,
            keypair,
            token,
        }
    }

    pub fn from(keypair: Keypair, token: Pubkey) -> Self {
        Self {
            client: RpcClient::new(RPC_URL),
            keypair,
            token,
        }
    }

    pub fn wallet_address(&self) -> String {
        self.keypair.pubkey().to_string()
    }

    pub fn token_address(&self) -> String {
        self.token.to_string()
    }

    pub fn get_keypair_base58(&self) -> String {
        self.keypair.to_base58_string()
    }
}

async fn wait_for_sol(client: &RpcClient, pubkey: &Pubkey) {
    loop {
        sleep(Duration::from_secs(30)).await;
        if let Ok(balance) = client.get_balance(pubkey) {
            if balance > 1_000_000_000 {
                return;
            }
        }
    }
}

async fn create_token(client: &RpcClient, keypair: &Keypair) -> Pubkey {
    wait_for_sol(client, &keypair.pubkey()).await;
    todo!();
}
